package com.stanfy.android.gradle.robolectric

import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.UnknownTaskException
import org.gradle.api.plugins.JavaPlugin
import org.slf4j.Logger
import org.slf4j.LoggerFactory

/**
 * Plugin for Android libraries that are packaged as a simple jar and use Robolectric for tests.
 */
class JavaRobolectricPlugin implements Plugin<Project> {
  
  private static final Logger LOG = LoggerFactory.getLogger('JavaRobolectricPlugin')
  
  private static void writeTo(String path, String value) {
    writeTo new File(path), value
  }
  
  private static void writeTo(File f, String value) {
    f.delete()
    f.parentFile.mkdirs()
    f << value
  }

  private static void genSources(Project project) {
    def pckg = project.javarob.packageName
    if (!pckg) { throw new RuntimeException("Package name is not defined") }
    def pckgPath = pckg.replaceAll(/\./, '/')

    final def nl = System.getProperty("line.separator");
    
    // R file
    def rFilePath = "${project.buildDir}/tmp/android/gen/$pckgPath/R.java"
    LOG.info "Create R file at $rFilePath"
    writeTo rFilePath, "package $pckg;$nl/** Android R file. */${nl}public class R { /* fake */ }$nl"
    
    // Manifest
    def mFilePath = "${project.buildDir}/tmp/android/AndroidManifest.xml"
    LOG.info "Create manifest file at $mFilePath"
    writeTo mFilePath, ("<?xml version=\"1.0\" encoding=\"utf-8\"?>$nl"
        + "<manifest xmlns:android=\"http://schemas.android.com/apk/res/android\"$nl"
        + "          package=\"$pckg\"$nl"
        + "          android:versionCode=\"1\" android.versionName=\"0\">$nl"
        + "  <appliaction/>$nl"
        + "</manifest>$nl")
    
    // res, assets folders
    new File("${project.buildDir}/tmp/android/res").mkdirs()
    new File("${project.buildDir}/tmp/android/assets").mkdirs()
  }
  
  private static File getRunnerFile(Project project) {
    def pckg = project.javarob.packageName
    if (!pckg) { throw new RuntimeException("Package name is not defined") }
    def pckgPath = pckg.replaceAll(/\./, '/')

    def baseDir = project.sourceSets.test.java.srcDirs.iterator().next()
    return new File(baseDir, "$pckgPath/test/Runner.java")
  }
  
  private static canTouchRunner(Project project) {
    File f = getRunnerFile(project)
    return !f.exists() || f.getText("UTF-8").contains("AUTOGENERATED")
  }
  
  static void makeTestRunner(Project project) {
    def template = (new groovy.text.SimpleTemplateEngine()).createTemplate(new InputStreamReader(
      JavaRobolectricPlugin.class.getResourceAsStream("Runner.tmpl"),
      "UTF-8"
    ))
    
    def relativeBuildDir = project.buildDir.absolutePath
    if (relativeBuildDir.startsWith(project.projectDir.absolutePath)) {
      StringBuilder sb = new StringBuilder(relativeBuildDir)
      sb.delete 0, project.projectDir.absolutePath.length()
      if (sb.charAt(0) == '/') {
        sb.delete 0, 1
      }
      relativeBuildDir = sb.toString() 
    }
    
    writeTo getRunnerFile(project), template.make([
      pckg : "${project.javarob.packageName}.test",
      basedir : "${relativeBuildDir}/tmp/android"
    ]).toString()
  }
  
  private static void makeTestRunnerIfCan(project) {
    if (canTouchRunner(project)) {
      makeTestRunner project
    }
  }
  
  @Override
  void apply(Project project) {

    // aply java plugin
    project.apply plugin : JavaPlugin
        
    // define our config
    project.extensions.create("javarob", JavaRobSettings)
    
    // generate fake R file, Manifest, and res folder
    // generate Runner if does not exist
    project.compileTestJava.doFirst {
      genSources project
      makeTestRunnerIfCan(project)
    }
    
    // add out sources to sourse sets
    project.sourceSets.test.java.srcDirs += new File("${project.buildDir}/tmp/android/gen")
    
    // add 'test-runner' task
    project.task('test-runner') << {
      makeTestRunner(project)
    }

    // create test runner before importing to IDE
    ['eclipse', 'idea'].each {
      def task
      try {
        task = project.tasks.getByName(it)
      } catch (UnknownTaskException e) {
        task = null
      }
      if (task) {
        task << {
          makeTestRunnerIfCan(project)
        }
      }
    }
    
  }
  
}
